--
--	WhenLevel
--	Written by scott@mithrandir.com
--
--	Version History
--		001	Initial version that records when leveling is done.
--		002	Added command line to output leveling information.
--

WhenLevel_File_Version = 002

if nil == WhenLevel or WhenLevel.versionNumber < WhenLevel_File_Version then

	WhenLevelData = { }

	WhenLevel = {

		versionNumber = WhenLevel_File_Version,
		currentlyProcessingLevel,
		eventDispatch = {			-- table of functions whose keys are the events
			['PLAYER_ENTERING_WORLD'] = function(self)
				self.currentlyProcessingLevel = UnitLevel("player")
				this:RegisterEvent("PLAYER_LEVEL_UP")
				this:RegisterEvent("TIME_PLAYED_MSG")
			end,
			['PLAYER_LEVEL_UP'] = function(self, arg1)
				--	arg1 tells us the new level, but we just cause another event to be posted
				self.currentlyProcessingLevel = arg1
				RequestTimePlayed()
			end,
			['TIME_PLAYED_MSG'] = function(self, arg1, arg2)
				--	arg1 tells us total time
				--	arg2 tells us total time at level
				local currentLevel = self.currentlyProcessingLevel
				if (nil == WhenLevelData[currentLevel]) then
					local hour, minute = GetGameTime()
					local weekday, month, day, year = CalendarGetDate()
					WhenLevelData[currentLevel] = { }
					WhenLevelData[currentLevel]["totalPlayed"] = arg1
					WhenLevelData[currentLevel]["serverTime"] = string.format("%4d-%02d-%02d %02d:%02d", year, month, day, hour, minute)
				end
			end,
			['VARIABLES_LOADED'] = function(self)
				--	Ensure the tooltip is not messed up
				if not com_mithrandir_whenLevelTooltip:IsOwned(UIParent) then
					com_mithrandir_whenLevelTooltip:SetOwner(UIParent, "ANCHOR_NONE")
				end
				this:RegisterEvent("PLAYER_ENTERING_WORLD")
			end,
			},

		FormattedSeconds = function(self, totalSeconds)
			local days = math.floor(totalSeconds / 86400)
			local remainder = totalSeconds - (days * 86400)
			local hours = math.floor(remainder / 3600)
			remainder = remainder - (hours * 3600)
			local minutes = math.floor(remainder / 60)
			local seconds = remainder - (minutes * 60)
			local retval = ""

			if (days > 0) then retval = retval .. days .. "d" end
			if (hours > 0 or days > 0) then retval = retval .. hours .. "h" end
			if (minutes > 0 or hours > 0 or days > 0) then retval = retval .. minutes .. "m" end
			retval = retval .. seconds .. "s"
			return retval
		end,

		OutputLevelMessage = function(self, level)
			local outputMessage = "Level " .. level .. " on " .. WhenLevelData[level]["serverTime"]
			if (nil ~= WhenLevelData[level + 1]) then
				local totalSeconds = WhenLevelData[level + 1]["totalPlayed"] - WhenLevelData[level]["totalPlayed"]
				outputMessage = outputMessage .. " played "  .. self:FormattedSeconds(totalSeconds) .. " [" .. self:FormattedSeconds(WhenLevelData[level + 1]["totalPlayed"]) .. "]"
			end
			DEFAULT_CHAT_FRAME:AddMessage(outputMessage)
		end,

		SlashCommand = function(self, msg)
			local firstLevelEncountered = nil
			for i = 1, 80, 1 do
				if (nil ~= WhenLevelData[i]) then
					if (nil == firstLevelEncountered) then
						firstLevelEncountered = i
						if (1 == i) then
							self:OutputLevelMessage(i)
						else
							DEFAULT_CHAT_FRAME:AddMessage("WhenLevel first used at Level " .. firstLevelEncountered)
						end
					else
						self:OutputLevelMessage(i)
					end
					
				end
			end
		end,

		Tooltip_OnEvent = function(self)
			if self.eventDispatch[event] then
				self.eventDispatch[event](self, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
			end
		end,

		Tooltip_OnLoad = function(self)
			SlashCmdList["WHENLEVEL"] = function(msg)
				self:SlashCommand(msg)
			end
			SLASH_WHENLEVEL1 = "/whenlevel"
			this:SetScript("OnEvent", function() self:Tooltip_OnEvent() end)
			this:RegisterEvent("VARIABLES_LOADED")
		end,

		}

end
